package com.sun.org.apache.bcel.internal.util;

import java.util.HashSet;
import com.sun.org.apache.bcel.internal.classfile.Attribute;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import com.sun.org.apache.bcel.internal.classfile.Utility;
import com.sun.org.apache.bcel.internal.classfile.ClassParser;
import java.io.File;
import java.io.IOException;
import com.sun.org.apache.bcel.internal.classfile.Method;
import java.util.Set;
import com.sun.org.apache.bcel.internal.classfile.ConstantPool;
import com.sun.org.apache.bcel.internal.classfile.JavaClass;

public class Class2HTML
{
    private final JavaClass java_class;
    private final String dir;
    private static String class_package;
    private static String class_name;
    private static ConstantPool constant_pool;
    private static final Set<String> basic_types;
    
    public Class2HTML(JavaClass java_class, String dir) throws IOException {
        Method[] methods = java_class.getMethods();
        this.java_class = java_class;
        this.dir = dir;
        Class2HTML.class_name = java_class.getClassName();
        Class2HTML.constant_pool = java_class.getConstantPool();
        int index = Class2HTML.class_name.lastIndexOf(46);
        if (index > -1) {
            Class2HTML.class_package = Class2HTML.class_name.substring(0, index);
        }
        else {
            Class2HTML.class_package = "";
        }
        ConstantHTML constant_html = new ConstantHTML(dir, Class2HTML.class_name, Class2HTML.class_package, methods, Class2HTML.constant_pool);
        AttributeHTML attribute_html = new AttributeHTML(dir, Class2HTML.class_name, Class2HTML.constant_pool, constant_html);
        new MethodHTML(dir, Class2HTML.class_name, methods, java_class.getFields(), constant_html, attribute_html);
        this.writeMainHTML(attribute_html);
        new CodeHTML(dir, Class2HTML.class_name, methods, Class2HTML.constant_pool, constant_html);
        attribute_html.close();
    }
    
    public static void main(String[] argv) throws IOException {
        String[] file_name = new String[argv.length];
        int files = 0;
        ClassParser parser = null;
        JavaClass java_class = null;
        String zip_file = null;
        char sep = File.separatorChar;
        String dir = "." + sep;
        for (int i = 0; i < argv.length; ++i) {
            if (argv[i].charAt(0) == '-') {
                if (argv[i].equals("-d")) {
                    dir = argv[++i];
                    if (!dir.endsWith("" + sep)) {
                        dir += sep;
                    }
                    File store = new File(dir);
                    if (!store.isDirectory()) {
                        boolean created = store.mkdirs();
                        if (!created && !store.isDirectory()) {
                            System.out.println("Tried to create the directory " + dir + " but failed");
                        }
                    }
                }
                else if (argv[i].equals("-zip")) {
                    zip_file = argv[++i];
                }
                else {
                    System.out.println("Unknown option " + argv[i]);
                }
            }
            else {
                file_name[files++] = argv[i];
            }
        }
        if (files == 0) {
            System.err.println("Class2HTML: No input files specified.");
        }
        else {
            for (int i = 0; i < files; ++i) {
                System.out.print("Processing " + file_name[i] + "...");
                if (zip_file == null) {
                    parser = new ClassParser(file_name[i]);
                }
                else {
                    parser = new ClassParser(zip_file, file_name[i]);
                }
                java_class = parser.parse();
                new Class2HTML(java_class, dir);
                System.out.println("Done.");
            }
        }
    }
    
    static String referenceClass(int index) {
        String str = Class2HTML.constant_pool.getConstantString(index, (byte)7);
        str = Utility.compactClassName(str);
        str = Utility.compactClassName(str, Class2HTML.class_package + ".", true);
        return "<A HREF=\"" + Class2HTML.class_name + "_cp.html#cp" + index + "\" TARGET=ConstantPool>" + str + "</A>";
    }
    
    static String referenceType(String type) {
        String short_type = Utility.compactClassName(type);
        short_type = Utility.compactClassName(short_type, Class2HTML.class_package + ".", true);
        int index = type.indexOf(91);
        String base_type = type;
        if (index > -1) {
            base_type = type.substring(0, index);
        }
        if (Class2HTML.basic_types.contains(base_type)) {
            return "<FONT COLOR=\"#00FF00\">" + type + "</FONT>";
        }
        return "<A HREF=\"" + base_type + ".html\" TARGET=_top>" + short_type + "</A>";
    }
    
    static String toHTML(String str) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < str.length(); ++i) {
            char ch;
            switch (ch = str.charAt(i)) {
                case '<': {
                    buf.append("&lt;");
                    break;
                }
                case '>': {
                    buf.append("&gt;");
                    break;
                }
                case '\n': {
                    buf.append("\\n");
                    break;
                }
                case '\r': {
                    buf.append("\\r");
                    break;
                }
                default: {
                    buf.append(ch);
                    break;
                }
            }
        }
        return buf.toString();
    }
    
    private void writeMainHTML(AttributeHTML attribute_html) throws IOException {
        try (final PrintWriter file = new PrintWriter(new FileOutputStream(this.dir + Class2HTML.class_name + ".html"))) {
            file.println("<HTML>\n<HEAD><TITLE>Documentation for " + Class2HTML.class_name + "</TITLE></HEAD>\n<FRAMESET BORDER=1 cols=\"30%,*\">\n<FRAMESET BORDER=1 rows=\"80%,*\">\n<FRAME NAME=\"ConstantPool\" SRC=\"" + Class2HTML.class_name + "_cp.html\"\n MARGINWIDTH=\"0\" MARGINHEIGHT=\"0\" FRAMEBORDER=\"1\" SCROLLING=\"AUTO\">\n<FRAME NAME=\"Attributes\" SRC=\"" + Class2HTML.class_name + "_attributes.html\"\n MARGINWIDTH=\"0\" MARGINHEIGHT=\"0\" FRAMEBORDER=\"1\" SCROLLING=\"AUTO\">\n</FRAMESET>\n<FRAMESET BORDER=1 rows=\"80%,*\">\n<FRAME NAME=\"Code\" SRC=\"" + Class2HTML.class_name + "_code.html\"\n MARGINWIDTH=0 MARGINHEIGHT=0 FRAMEBORDER=1 SCROLLING=\"AUTO\">\n<FRAME NAME=\"Methods\" SRC=\"" + Class2HTML.class_name + "_methods.html\"\n MARGINWIDTH=0 MARGINHEIGHT=0 FRAMEBORDER=1 SCROLLING=\"AUTO\">\n</FRAMESET></FRAMESET></HTML>");
        }
        Attribute[] attributes = this.java_class.getAttributes();
        for (int i = 0; i < attributes.length; ++i) {
            attribute_html.writeAttribute(attributes[i], "class" + i);
        }
    }
    
    static {
        (basic_types = new HashSet()).add("int");
        Class2HTML.basic_types.add("short");
        Class2HTML.basic_types.add("boolean");
        Class2HTML.basic_types.add("void");
        Class2HTML.basic_types.add("char");
        Class2HTML.basic_types.add("byte");
        Class2HTML.basic_types.add("long");
        Class2HTML.basic_types.add("double");
        Class2HTML.basic_types.add("float");
    }
}
